/**
 * @file graphicsScan.C
 * @author Rafael Arco Arredondo <<a href="mailto:rarco@correo.ugr.es">rarco@correo.ugr.es</a>>
 * @brief File with functions to plot several graphics from the ROOT @c TTree contained in file
 * @c scan_summary.root.
 */

#include "Riostream.h"
#include "TROOT.h"
#include "TFile.h"
#include "TTree.h"
#include "TGraph.h"
#include "TCanvas.h"
#include "TLegend.h"
#include "TMultiGraph.h"
#include "TStyle.h"
#include "TEventList.h"
#include "TCollection.h"
#include "TPaveStats.h"
#include "TF2.h"

inline UInt_t numOrientation(Char_t orientation[]) {
  if(strcmp("r3", orientation) == 0 || strcmp("r4", orientation) == 0)
    return (UInt_t) (orientation[1] - '0' - 1);

  else 
    return (UInt_t) (orientation[1] - '0');
}

/**
 * @brief This function fills an array of @c Char_t* with the name of the different sites in
 * @c tree, and returns ther number of sites.
 * 
 * This function fills an array of @c Char_t* with the name of the different sites in
 * @c tree, and returns ther number of sites. The names in the tree must have less than four
 * characters (this is the current format: {ALU, BMC, etc.}).
 * 
 * @param tree ROOT @c TTree from which the names of the sites are read.
 * @retval siteNames Array of @c Char_t* which will contain the names of the sites
 * @return The number of sites in the @c tree.
 */
UInt_t fillSites(TTree* tree, Char_t siteNames[][4]) {
  Char_t site[4];
  UInt_t numEntries, numSites;

  tree -> SetBranchAddress("Site", site);

  // We fill the array siteNames

  numEntries = tree -> GetEntries();

  tree -> GetEntry(0);

  strcpy(siteNames[0], site);

  numSites = 1;

  for(UInt_t i = 1; i < numEntries; i++) {
    tree -> GetEntry(i);
    
    if(strcmp(site, siteNames[numSites - 1]) != 0) {
      strcpy(siteNames[numSites], site);
      numSites++;
    }
  }

  return numSites;
}

/**
 * @brief Function to plot 2-dimensional graphics showing one parameter vs poutre position.
 * 
 * This function plots the 2-dimensional plots for all the sites for a specific parameter versus
 * the poutre position (each plot corresponds to a specific chamber). It stores the plots both
 * in ROOT format, in order to load them from the ROOT CINT C++ interpreter, and in EPS format.
 * 
 * The function creates the following hierarchy of directories:
 * <tt>
 * poutre-root-files/&lt;site&gt;/&lt;serial-number&gt;/&lt;graphName&gt;.root
 * poutre-graphics/&lt;site&gt;/&lt;serial-number&gt;/&lt;graphName&gt;.eps
 * </tt>
 *
 * @param tree ROOT @c TTree from where the plotted data are extracted.
 * @param siteNames Array with the names of the different sites. The names must have less than
 * four letters (this is the current format: {ALU, BMC, etc.}). This array can be filled with
 * the function @c fillSites.
 * @param numSites The number of sites in @c tree.
 * @param parameter Parameter that will be plotted. One of:
 * - @c TempAvg
 * - @c TempVar
 * - @c TempStdDev
 * - @c DeltaY
 * - @c DeltaZ
 * - @c DeltaAlpha
 * - @c ZPitch
 * - @c YPitch
 * - @c TomoPrecision
 * @param graphName Name the plotted graph will have.
 * @param title Title the plotted graph will have.
 * @param unit Units that were used to measure the plotted parameter.
 */
void twoDimPlotPoutre(TTree* tree, Char_t siteNames[][4], UInt_t numSites, Char_t parameter[],
		      Char_t graphName[], Char_t title[], Char_t unit[]) {
  Char_t serialNumberChar[5];
  Char_t drawCommand[256];
  Char_t graphNameComplete[256];
  Char_t graphTitle[256];
  Char_t axisTitle[256];
  Char_t printName[256];
  Char_t cutCommand[256];
  UInt_t numChambers, indexSite, indexChamber, numEntries;
  UInt_t serialNumber, serialNumbers[50];
  TEventList* myList;
  TFile* rootFile;
  Char_t rootFileName[256];
  TGraph *graph0 = 0, *graph1 = 0, *graph3 = 0, *graph4 = 0, *graph5 = 0, *graph6 = 0,
    *graph8 = 0, *graph9 = 0;
  TMultiGraph* mg;
  TLegend* leg;
  Double_t nomValue;
  Double_t upperLimit = 0, lowerLimit = 0;
  Char_t nomValueParameter[256];
  Char_t dirCommand[256];
  Bool_t dataR0 = false, dataR1 = false, dataR3 = false, dataR4 = false, dataR5 = false,
    dataR6 = false, dataR8 = false, dataR9 = false;
  UInt_t highestMeasurId;
  Char_t highestMeasurIdChar[5];

  // This canvas will contain the histograms of all chambers for a particular site)
  TCanvas* canvas1 = new TCanvas("canvas1");

  canvas1 -> SetLeftMargin(0.18);

  tree -> SetBranchAddress("SerialNumber", &serialNumber);

  // Loop around all sites
  for(indexSite = 0; indexSite < numSites; indexSite++) {  
    // First we calculate how many chambers the tree has for the current site, and store the
    // serial numbers of them
    sprintf(cutCommand, "site == \"%s\"", siteNames[indexSite]);

    cout << siteNames[indexSite] << endl;
    cout << cutCommand << endl;
        
    tree -> Draw(">>myList", cutCommand);

//     myList = (TEventList*) gDirectory -> Get("myList");

//     tree -> SetEventList(myList);  
    
//     // We fill the array serialNumbers

//     numEntries = myList -> GetN();

//     tree -> GetEntry(myList -> GetEntry(0));

//     serialNumbers[0] = serialNumber;

//     numChambers = 1;

//     for(UInt_t i = 1; i < numEntries; i++) {
//       tree -> GetEntry(myList -> GetEntry(i));
 
//       if(serialNumber != serialNumbers[numChambers - 1]) {
// 	serialNumbers[numChambers] = serialNumber;
//  	numChambers++;
//       }
//     }

//     tree -> SetEventList(0);

    // Once we know the serial numbers of the chambers for the current site, we can begin
    // plotting one graph per chamber
//     for(indexChamber = 0; indexChamber < numChambers; indexChamber++) {

//       // We create a hierarchy of directories to hold the .eps and .root files
//       strcpy(dirCommand, "mkdir -p poutre-graphics/");
//       strcat(dirCommand, siteNames[indexSite]);
//       strcat(dirCommand, "/");
//       sprintf(serialNumberChar, "%d", serialNumbers[indexChamber]);
//       strcat(dirCommand, serialNumberChar);
//       system(dirCommand);

//       strcpy(dirCommand, "mkdir -p poutre-root-files/");
//       strcat(dirCommand, siteNames[indexSite]);
//       strcat(dirCommand, "/");
//       strcat(dirCommand, serialNumberChar);
//       system(dirCommand);

//       // We set the plot scale (this might be changed in case the tree has more records)
//       if(strcmp("DeltaY", parameter) == 0 || strcmp("YPitch", parameter) == 0
// 	 || strcmp("ZPitch", parameter) == 0) {
// 	strcpy(cutCommand, "site == \"");
// 	strcat(cutCommand, siteNames[indexSite]);
// 	strcat(cutCommand, "\" && serialNumber == ");
// 	strcat(cutCommand, serialNumberChar);

// 	strcpy(nomValueParameter, "nom");
// 	strcat(nomValueParameter, parameter);

// 	tree -> Draw(nomValueParameter, cutCommand, "goff");
// 	nomValue = tree -> GetV1()[0];

// 	if(strcmp("DeltaY", parameter) == 0) {
// 	  if(strcmp(siteNames[indexSite], "THE") != 0) {
// 	    upperLimit = nomValue + 0.1;
// 	    lowerLimit = nomValue - 0.1;
// 	  }

// 	  else {
// 	    upperLimit = nomValue + 0.03;
// 	    lowerLimit = nomValue - 0.03;
// 	  }
// 	}

// 	else if(strcmp("ZPitch", parameter) == 0) {
// 	  upperLimit = nomValue + 0.0025;
// 	  lowerLimit = nomValue - 0.0025;
// 	}

// 	else { // Y Pitch
// 	  if(strcmp(siteNames[indexSite], "THE") != 0) {
// 	    upperLimit = nomValue + 0.025;
// 	    lowerLimit = nomValue - 0.025;
// 	  }

// 	  else {
// 	    upperLimit = nomValue + 0.015;
// 	    lowerLimit = nomValue - 0.015;
// 	  }
// 	}
//       }

//       else {
// 	if(strcmp("TempAvg", parameter) == 0) {
// 	  upperLimit = 21;
// 	  lowerLimit = 19;
// 	}

// 	else if(strcmp("TempVar", parameter) == 0) {
// 	  upperLimit = 0.14;
// 	  lowerLimit = 0;
// 	}

// 	else if(strcmp("TempStdDev", parameter) == 0) {
// 	  upperLimit = 0.24;
// 	  lowerLimit = 0;
// 	}

// 	else if(strcmp("ILDistance", parameter) == 0) {
// 	  upperLimit = 0.05;
// 	  lowerLimit = -0.07;
// 	}

// 	else if(strcmp("DeltaZ", parameter) == 0) {
// 	  upperLimit = 0.1;
// 	  lowerLimit = -0.1;
// 	}

// 	else { // DeltaAlpha
// 	  if(strcmp(siteNames[indexSite], "THE") != 0) {
// 	    upperLimit = 100;
// 	    lowerLimit = -100;
// 	  }

// 	  else {
// 	    upperLimit = 40;
// 	    lowerLimit = -40;
// 	  }
// 	}
//       }

//       // We set the name of the graph, including the serial number
//       strcpy(graphNameComplete, graphName);
//       strcat(graphNameComplete, "-");
//       strcat(graphNameComplete, siteNames[indexSite]);
//       strcat(graphNameComplete, "-");
//       sprintf(serialNumberChar, "%d", serialNumbers[indexChamber]);
//       strcat(graphNameComplete, serialNumberChar);

//       // We set the first argument for the TTree::Draw member function (the parameter we want
//       // to plot)
//       strcpy(drawCommand, parameter);
//       strcat(drawCommand, ":poutre");

//       // We set the name for the .root file and open it
//       strcpy(rootFileName, "./poutre-root-files/");
//       strcat(rootFileName, siteNames[indexSite]);
//       strcat(rootFileName, "/");
//       strcat(rootFileName, serialNumberChar);
//       strcat(rootFileName, "/");
//       strcat(rootFileName, graphNameComplete);
//       strcat(rootFileName, ".root");
//       rootFile = new TFile(rootFileName, "RECREATE");

//       // We set the graph title
//       strcpy(graphTitle, title);
//       strcat(graphTitle, " (");
//       strcat(graphTitle, siteNames[indexSite]);
//       strcat(graphTitle, "-");
//       strcat(graphTitle, serialNumberChar);
//       strcat(graphTitle, ")");

//       // We set the axis title
//       strcpy(axisTitle, title);
//       strcat(axisTitle, " (");
//       strcat(axisTitle, unit);
//       strcat(axisTitle, ")");

//       // We initialise the TMultiGraph object which will hold the different datasets we will use
//       // and the TLegend object to display the meaning of the points in the graph
//       mg = new TMultiGraph(graphNameComplete, graphTitle);
//       leg = new TLegend(0.79, 0.11, 0.89, 0.11);
//       leg -> SetFillColor(10);

//       // Plots for all parameters except Z Pitch and Y Pitch, which are a bit different because
//       // they have 2 values (one per multilayer) instead of 1
//       if(strcmp("ZPitch", parameter) != 0 && strcmp("YPitch", parameter) != 0) {
// 	// We set the second argument for the TTree:Draw member function (the selection criteria)
// 	strcpy(cutCommand, "site == \"");
// 	strcat(cutCommand, siteNames[indexSite]);
// 	strcat(cutCommand, "\" && serialNumber == ");
// 	sprintf(serialNumberChar, "%d", serialNumbers[indexChamber]);
// 	strcat(cutCommand, serialNumberChar);
// 	strcat(cutCommand, " && orientation == \"r0\"");

// 	// We must select the highest measurement id in order to plot only these values
// 	tree -> Draw("MeasurementId", cutCommand);
// 	highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
// 	sprintf(highestMeasurIdChar, "%d", highestMeasurId);
// 	strcat(cutCommand, " && measurementId == ");
// 	strcat(cutCommand, highestMeasurIdChar);
// 	strcat(cutCommand, " && badScan == 0");

// 	// We create the graph
// 	tree -> Draw(drawCommand, cutCommand, "goff");

// 	if((dataR0 = (tree -> GetSelectedRows() > 0))) {
// 	  graph0 = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
// 	  graph0 -> SetMarkerStyle(20);
// 	  graph0 -> SetMarkerSize(1);
// 	  graph0 -> SetMarkerColor(kRed);

// 	  if(strcmp(parameter, "DeltaY") == 0 || strcmp(parameter, "DeltaZ") == 0) {
// 	    graph0 -> Fit("pol1", "QM");
// 	    TF1* f0 = graph0 -> GetFunction("pol1");
// 	    f0 -> SetLineColor(kRed);
// 	  }

// 	  mg -> Add(graph0);

// 	  leg -> SetY2(leg -> GetY2() + 0.05);
// 	  leg -> AddEntry(graph0, "r0", "p");
// 	}

// 	// We set the second argument for the TTree:Draw member function (the selection criteria)
// 	strcpy(cutCommand, "site == \"");
// 	strcat(cutCommand, siteNames[indexSite]);
// 	strcat(cutCommand, "\" && serialNumber == ");
// 	strcat(cutCommand, serialNumberChar);
// 	strcat(cutCommand, " && orientation == \"r1\"");

// 	// We must select the highest measurement id in order to plot only these values
// 	tree -> Draw("MeasurementId", cutCommand);
// 	highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
// 	sprintf(highestMeasurIdChar, "%d", highestMeasurId);
// 	strcat(cutCommand, " && measurementId == ");
// 	strcat(cutCommand, highestMeasurIdChar);
// 	strcat(cutCommand, " && badScan == 0");

// 	// We create the graph
// 	tree -> Draw(drawCommand, cutCommand, "goff");

// 	if((dataR1 = (tree -> GetSelectedRows() > 0))) {
// 	  graph1 = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
// 	  graph1 -> SetMarkerStyle(21);
// 	  graph1 -> SetMarkerSize(1);
// 	  graph1 -> SetMarkerColor(kBlue);

// 	  if(strcmp(parameter, "DeltaY") == 0 || strcmp(parameter, "DeltaZ") == 0) {
// 	    graph1 -> Fit("pol1", "QM");
// 	    TF1* f1 = graph1 -> GetFunction("pol1");
// 	    f1 -> SetLineColor(kBlue);
// 	  }

// 	  mg -> Add(graph1);
// 	  leg -> SetY2(leg -> GetY2() + 0.05);
// 	  leg -> AddEntry(graph1, "r1", "p");
// 	}

// 	// We set the second argument for the TTree:Draw member function (the selection criteria)
// 	strcpy(cutCommand, "site == \"");
// 	strcat(cutCommand, siteNames[indexSite]);
// 	strcat(cutCommand, "\" && serialNumber == ");
// 	strcat(cutCommand, serialNumberChar);
// 	strcat(cutCommand, " && orientation == \"r3\"");

// 	// We must select the highest measurement id in order to plot only these values
// 	tree -> Draw("MeasurementId", cutCommand);
// 	highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
// 	sprintf(highestMeasurIdChar, "%d", highestMeasurId);
// 	strcat(cutCommand, " && measurementId == ");
// 	strcat(cutCommand, highestMeasurIdChar);
// 	strcat(cutCommand, " && badScan == 0");

// 	// We create the graph
// 	tree -> Draw(drawCommand, cutCommand, "goff");

// 	if((dataR3 = (tree -> GetSelectedRows() > 0))) {
// 	  graph3 = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
// 	  graph3 -> SetMarkerStyle(22);
// 	  graph3 -> SetMarkerSize(1);
// 	  graph3 -> SetMarkerColor(kGreen);

// 	  if(strcmp(parameter, "DeltaY") == 0 || strcmp(parameter, "DeltaZ") == 0) {
// 	    graph3 -> Fit("pol1", "QM");
// 	    TF1* f3 = graph3 -> GetFunction("pol1");
// 	    f3 -> SetLineColor(kGreen);
// 	  }

// 	  mg -> Add(graph3);
// 	  leg -> SetY2(leg -> GetY2() + 0.05);
// 	  leg -> AddEntry(graph3, "r3", "p");
// 	}

// 	// We set the second argument for the TTree:Draw member function (the selection criteria)
// 	strcpy(cutCommand, "site == \"");
// 	strcat(cutCommand, siteNames[indexSite]);
// 	strcat(cutCommand, "\" && serialNumber == ");
// 	strcat(cutCommand, serialNumberChar);
// 	strcat(cutCommand, " && orientation == \"r4\"");

// 	// We must select the highest measurement id in order to plot only these values
// 	tree -> Draw("MeasurementId", cutCommand);
// 	highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
// 	sprintf(highestMeasurIdChar, "%d", highestMeasurId);
// 	strcat(cutCommand, " && measurementId == ");
// 	strcat(cutCommand, highestMeasurIdChar);
// 	strcat(cutCommand, " && badScan == 0");

// 	// We create the graph
// 	tree -> Draw(drawCommand, cutCommand, "goff");

// 	if((dataR4 = (tree -> GetSelectedRows() > 0))) {
// 	  graph4 = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
// 	  graph4 -> SetMarkerStyle(23);
// 	  graph4 -> SetMarkerSize(1);
// 	  graph4 -> SetMarkerColor(kYellow);

// 	  if(strcmp(parameter, "DeltaY") == 0 || strcmp(parameter, "DeltaZ") == 0) {
// 	    graph4 -> Fit("pol1", "QM");
// 	    TF1* f4 = graph4 -> GetFunction("pol1");
// 	    f4 -> SetLineColor(kYellow);
// 	  }

// 	  mg -> Add(graph4);
// 	  leg -> SetY2(leg -> GetY2() + 0.05);
// 	  leg -> AddEntry(graph4, "r4", "p");
// 	}
//       }

//       else {
// 	// We set the second argument for the TTree:Draw member function (the selection criteria)
// 	strcpy(cutCommand, "site == \"");
// 	strcat(cutCommand, siteNames[indexSite]);
// 	strcat(cutCommand, "\" && serialNumber == ");
// 	sprintf(serialNumberChar, "%d", serialNumbers[indexChamber]);
// 	strcat(cutCommand, serialNumberChar);
// 	strcat(cutCommand, " && orientation == \"r0\" && ml == 1");

// 	// We must select the highest measurement id in order to plot only these values
// 	tree -> Draw("MeasurementId", cutCommand);
// 	highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
// 	sprintf(highestMeasurIdChar, "%d", highestMeasurId);
// 	strcat(cutCommand, " && measurementId == ");
// 	strcat(cutCommand, highestMeasurIdChar);
// 	strcat(cutCommand, " && badScan == 0");

// 	// We create the graph
// 	tree -> Draw(drawCommand, cutCommand, "goff");

// 	if((dataR0 = (tree -> GetSelectedRows() > 0))) {
// 	  graph0 = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
// 	  graph0 -> SetMarkerStyle(20);
// 	  graph0 -> SetMarkerSize(1);
// 	  graph0 -> SetMarkerColor(kRed);

// 	  if(strcmp(parameter, "DeltaY") == 0 || strcmp(parameter, "DeltaZ") == 0) {
// 	    graph0 -> Fit("pol1", "QM");
// 	    TF1* f0 = graph0 -> GetFunction("pol1");
// 	    f0 -> SetLineColor(kRed);
// 	  }

// 	  mg -> Add(graph0);
// 	  leg -> SetY2(leg -> GetY2() + 0.05);
// 	  leg -> AddEntry(graph0, "ML1 r0", "p");
// 	}

// 	// We set the second argument for the TTree:Draw member function (the selection criteria)
// 	strcpy(cutCommand, "site == \"");
// 	strcat(cutCommand, siteNames[indexSite]);
// 	strcat(cutCommand, "\" && serialNumber == ");
// 	sprintf(serialNumberChar, "%d", serialNumbers[indexChamber]);
// 	strcat(cutCommand, serialNumberChar);
// 	strcat(cutCommand, " && orientation == \"r0\" && ml == 2");

// 	// We must select the highest measurement id in order to plot only these values
// 	tree -> Draw("MeasurementId", cutCommand);
// 	highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
// 	sprintf(highestMeasurIdChar, "%d", highestMeasurId);
// 	strcat(cutCommand, " && measurementId == ");
// 	strcat(cutCommand, highestMeasurIdChar);
// 	strcat(cutCommand, " && badScan == 0");

// 	// We create the graph
// 	tree -> Draw(drawCommand, cutCommand, "goff");

// 	if((dataR5 = (tree -> GetSelectedRows() > 0))) {
// 	  graph5 = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
// 	  graph5 -> SetMarkerStyle(24);
// 	  graph5 -> SetMarkerSize(1);
// 	  graph5 -> SetMarkerColor(kRed);

// 	  if(strcmp(parameter, "DeltaY") == 0 || strcmp(parameter, "DeltaZ") == 0) {
// 	    graph5 -> Fit("pol1", "QM");
// 	    TF1* f5 = graph5 -> GetFunction("pol1");
// 	    f5 -> SetLineColor(kRed);
// 	  }

// 	  mg -> Add(graph5);
// 	  leg -> SetY2(leg -> GetY2() + 0.05);
// 	  leg -> AddEntry(graph5, "ML2 r0", "p");
// 	}

// 	// We set the second argument for the TTree:Draw member function (the selection criteria)
// 	strcpy(cutCommand, "site == \"");
// 	strcat(cutCommand, siteNames[indexSite]);
// 	strcat(cutCommand, "\" && serialNumber == ");
// 	strcat(cutCommand, serialNumberChar);
// 	strcat(cutCommand, " && orientation == \"r1\" && ml == 1");

// 	// We must select the highest measurement id in order to plot only these values
// 	tree -> Draw("MeasurementId", cutCommand);
// 	highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
// 	sprintf(highestMeasurIdChar, "%d", highestMeasurId);
// 	strcat(cutCommand, " && measurementId == ");
// 	strcat(cutCommand, highestMeasurIdChar);
// 	strcat(cutCommand, " && badScan == 0");

// 	// We create the graph
// 	tree -> Draw(drawCommand, cutCommand, "goff");

// 	if((dataR1 = (tree -> GetSelectedRows() > 0))) {
// 	  graph1 = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
// 	  graph1 -> SetMarkerStyle(21);
// 	  graph1 -> SetMarkerSize(1);
// 	  graph1 -> SetMarkerColor(kBlue);

// 	  if(strcmp(parameter, "DeltaY") == 0 || strcmp(parameter, "DeltaZ") == 0) {
// 	    graph1 -> Fit("pol1", "QM");
// 	    TF1* f1 = graph1 -> GetFunction("pol1");
// 	    f1 -> SetLineColor(kBlue);
// 	  }

// 	  mg -> Add(graph1);
// 	  leg -> SetY2(leg -> GetY2() + 0.05);
// 	  leg -> AddEntry(graph1, "ML1 r1", "p");
// 	}

// 	// We set the second argument for the TTree:Draw member function (the selection criteria)
// 	strcpy(cutCommand, "site == \"");
// 	strcat(cutCommand, siteNames[indexSite]);
// 	strcat(cutCommand, "\" && serialNumber == ");
// 	strcat(cutCommand, serialNumberChar);
// 	strcat(cutCommand, " && orientation == \"r1\" && ml == 2");

// 	// We must select the highest measurement id in order to plot only these values
// 	tree -> Draw("MeasurementId", cutCommand);
// 	highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
// 	sprintf(highestMeasurIdChar, "%d", highestMeasurId);
// 	strcat(cutCommand, " && measurementId == ");
// 	strcat(cutCommand, highestMeasurIdChar);
// 	strcat(cutCommand, " && badScan == 0");

// 	// We create the graph
// 	tree -> Draw(drawCommand, cutCommand, "goff");

// 	if((dataR6 = (tree -> GetSelectedRows() > 0))) {
// 	  graph6 = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
// 	  graph6 -> SetMarkerStyle(25);
// 	  graph6 -> SetMarkerSize(1);
// 	  graph6 -> SetMarkerColor(kBlue);

// 	  if(strcmp(parameter, "DeltaY") == 0 || strcmp(parameter, "DeltaZ") == 0) {
// 	    graph6 -> Fit("pol1", "QM");
// 	    TF1* f6 = graph6 -> GetFunction("pol1");
// 	    f6 -> SetLineColor(kBlue);
// 	  }

// 	  mg -> Add(graph6);
// 	  leg -> SetY2(leg -> GetY2() + 0.05);
// 	  leg -> AddEntry(graph6, "ML2 r1", "p");
// 	}

// 	// We set the second argument for the TTree:Draw member function (the selection criteria)
// 	strcpy(cutCommand, "site == \"");
// 	strcat(cutCommand, siteNames[indexSite]);
// 	strcat(cutCommand, "\" && serialNumber == ");
// 	strcat(cutCommand, serialNumberChar);
// 	strcat(cutCommand, " && orientation == \"r3\" && ml == 1");

// 	// We must select the highest measurement id in order to plot only these values
// 	tree -> Draw("MeasurementId", cutCommand);
// 	highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
// 	sprintf(highestMeasurIdChar, "%d", highestMeasurId);
// 	strcat(cutCommand, " && measurementId == ");
// 	strcat(cutCommand, highestMeasurIdChar);
// 	strcat(cutCommand, " && badScan == 0");

// 	// We create the graph
// 	tree -> Draw(drawCommand, cutCommand, "goff");

// 	if((dataR3 = (tree -> GetSelectedRows()))) {
// 	  graph3 = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
// 	  graph3 -> SetMarkerStyle(22);
// 	  graph3 -> SetMarkerSize(1);
// 	  graph3 -> SetMarkerColor(kGreen);

// 	  if(strcmp(parameter, "DeltaY") == 0 || strcmp(parameter, "DeltaZ") == 0) {
// 	    graph3 -> Fit("pol1", "QM");
// 	    TF1* f3 = graph3 -> GetFunction("pol1");
// 	    f3 -> SetLineColor(kGreen);
// 	  }

// 	  mg -> Add(graph3);
// 	  leg -> SetY2(leg -> GetY2() + 0.05);
// 	  leg -> AddEntry(graph3, "ML1 r3", "p");
// 	}

// 	// We set the second argument for the TTree:Draw member function (the selection criteria)
// 	strcpy(cutCommand, "site == \"");
// 	strcat(cutCommand, siteNames[indexSite]);
// 	strcat(cutCommand, "\" && serialNumber == ");
// 	strcat(cutCommand, serialNumberChar);
// 	strcat(cutCommand, " && orientation == \"r3\" && ml == 2");

// 	// We must select the highest measurement id in order to plot only these values
// 	tree -> Draw("MeasurementId", cutCommand);
// 	highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
// 	sprintf(highestMeasurIdChar, "%d", highestMeasurId);
// 	strcat(cutCommand, " && measurementId == ");
// 	strcat(cutCommand, highestMeasurIdChar);
// 	strcat(cutCommand, " && badScan == 0");

// 	// We create the graph
// 	tree -> Draw(drawCommand, cutCommand, "goff");

// 	if((dataR8 = (tree -> GetSelectedRows() > 0))) {
// 	  graph8 = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
// 	  graph8 -> SetMarkerStyle(26);
// 	  graph8 -> SetMarkerSize(1);
// 	  graph8 -> SetMarkerColor(kGreen);

// 	  if(strcmp(parameter, "DeltaY") == 0 || strcmp(parameter, "DeltaZ") == 0) {
// 	    graph8 -> Fit("pol1", "QM");
// 	    TF1* f8 = graph8 -> GetFunction("pol1");
// 	    f8 -> SetLineColor(kGreen);
// 	  }

// 	  mg -> Add(graph8);
// 	  leg -> SetY2(leg -> GetY2() + 0.05);
// 	  leg -> AddEntry(graph8, "ML2 r3", "p");
// 	}

// 	// We set the second argument for the TTree:Draw member function (the selection criteria)
// 	strcpy(cutCommand, "site == \"");
// 	strcat(cutCommand, siteNames[indexSite]);
// 	strcat(cutCommand, "\" && serialNumber == ");
// 	strcat(cutCommand, serialNumberChar);
// 	strcat(cutCommand, " && orientation == \"r4\" && ml == 1");

// 	// We must select the highest measurement id in order to plot only these values
// 	tree -> Draw("MeasurementId", cutCommand);
// 	highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
// 	sprintf(highestMeasurIdChar, "%d", highestMeasurId);
// 	strcat(cutCommand, " && measurementId == ");
// 	strcat(cutCommand, highestMeasurIdChar);
// 	strcat(cutCommand, " && badScan == 0");

// 	// We create the graph
// 	tree -> Draw(drawCommand, cutCommand, "goff");

// 	if((dataR4 = (tree -> GetSelectedRows() > 0))) {
// 	  graph4 = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
// 	  graph4 -> SetMarkerStyle(23);
// 	  graph4 -> SetMarkerSize(1);
// 	  graph4 -> SetMarkerColor(kYellow);

// 	  if(strcmp(parameter, "DeltaY") == 0 || strcmp(parameter, "DeltaZ") == 0) {
// 	    graph4 -> Fit("pol1", "QM");
// 	    TF1* f4 = graph4 -> GetFunction("pol1");
// 	    f4 -> SetLineColor(kYellow);
// 	  }

// 	  mg -> Add(graph4);
// 	  leg -> SetY2(leg -> GetY2() + 0.05);
// 	  leg -> AddEntry(graph4, "ML1 r4", "p");
// 	}

// 	// We set the second argument for the TTree:Draw member function (the selection criteria)
// 	strcpy(cutCommand, "site == \"");
// 	strcat(cutCommand, siteNames[indexSite]);
// 	strcat(cutCommand, "\" && serialNumber == ");
// 	strcat(cutCommand, serialNumberChar);
// 	strcat(cutCommand, " && orientation == \"r4\" && ml == 2");

// 	// We must select the highest measurement id in order to plot only these values
// 	tree -> Draw("MeasurementId", cutCommand);
// 	highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
// 	sprintf(highestMeasurIdChar, "%d", highestMeasurId);
// 	strcat(cutCommand, " && measurementId == ");
// 	strcat(cutCommand, highestMeasurIdChar);
// 	strcat(cutCommand, " && badScan == 0");

// 	// We create the graph
// 	tree -> Draw(drawCommand, cutCommand, "goff");

// 	if((dataR9 = (tree -> GetSelectedRows() > 0))) {
// 	  graph9 = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
// 	  graph9 -> SetMarkerStyle(27);
// 	  graph9 -> SetMarkerSize(1);
// 	  graph9 -> SetMarkerColor(kYellow);

// 	  if(strcmp(parameter, "DeltaY") == 0 || strcmp(parameter, "DeltaZ") == 0) {
// 	    graph9 -> Fit("pol1", "QM");
// 	    TF1* f9 = graph9 -> GetFunction("pol1");
// 	    f9 -> SetLineColor(kYellow);
// 	  }

// 	  mg -> Add(graph9);
// 	  leg -> SetY2(leg -> GetY2() + 0.05);
// 	  leg -> AddEntry(graph9, "ML2 r4", "p");
// 	}
//       }

//       // We set the limits for the multigraph
//       mg -> SetMaximum(upperLimit);
//       mg -> SetMinimum(lowerLimit);


//       // We draw the multigraph and the legend
//       mg -> Draw("ap");
//       leg -> Draw();

//       // We set the titles for the multigraph
//       mg -> GetXaxis() -> SetTitle("Poutre (mm)");
//       mg -> GetYaxis() -> SetTitle(axisTitle);
//       mg -> GetYaxis() -> SetTitleOffset(1.7);

//       canvas1 -> Update();

//       // We set the position and colour for the stats boxes, if we have plotted the
//       // multilayer distance or the multilayer Z Shift (the stats give information about the
//       // linear fit which is performed for these two parameters).
//       if(strcmp(parameter, "DeltaY") == 0 || strcmp(parameter, "DeltaZ") == 0) {
// 	if(dataR0) {
// 	  TPaveStats* stats0 = (TPaveStats*) graph0 -> GetListOfFunctions()
// 	    -> FindObject("stats");
// 	  stats0 -> SetTextColor(kRed);
// 	  stats0 -> SetX1NDC(0.18);
// 	  stats0 -> SetX2NDC(0.38);
// 	  stats0 -> SetY1NDC(0.8);
// 	  stats0 -> SetY2NDC(0.9);
// 	  canvas1 -> Modified();
// 	}

// 	if(dataR1) {
// 	  TPaveStats* stats1 = (TPaveStats*) graph1 -> GetListOfFunctions()
// 	    -> FindObject("stats");
// 	  stats1 -> SetTextColor(kBlue); 
// 	  stats1 -> SetX1NDC(0.70);
// 	  stats1 -> SetX2NDC(0.90);
// 	  stats1 -> SetY1NDC(0.8);
// 	  stats1 -> SetY2NDC(0.9);
// 	  canvas1 -> Modified();
// 	}

// 	if(dataR3) {
// 	  TPaveStats* stats3 = (TPaveStats*) graph3 -> GetListOfFunctions()
// 	    -> FindObject("stats");
// 	  stats3 -> SetTextColor(kGreen); 
// 	  stats3 -> SetX1NDC(0.18);
// 	  stats3 -> SetX2NDC(0.38);
// 	  stats3 -> SetY1NDC(0.1);
// 	  stats3 -> SetY2NDC(0.2);
// 	  canvas1 -> Modified();
// 	}

// 	if(dataR4) {
// 	  TPaveStats* stats4 = (TPaveStats*) graph4 -> GetListOfFunctions()
// 	    -> FindObject("stats");
// 	  stats4 -> SetTextColor(kYellow); 
// 	  stats4 -> SetX1NDC(0.59);
// 	  stats4 -> SetX2NDC(0.79);
// 	  stats4 -> SetY1NDC(0.1);
// 	  stats4 -> SetY2NDC(0.2);
// 	  canvas1 -> Modified();
// 	}
//       }

//       // We write the multigraph and the legend to the .root file
//       mg -> Write();
//       leg -> Write();

//       // We set the name for the EPS file
//       strcpy(printName, "./poutre-graphics/");
//       strcat(printName, siteNames[indexSite]);
//       strcat(printName, "/");
//       strcat(printName, serialNumberChar);
//       strcat(printName, "/");
//       strcat(printName, graphNameComplete);
//       strcat(printName, ".eps");

//       // We print the graph to an EPS file
//       canvas1 -> Print(printName);

//       // We close the .root file
//       rootFile -> Close();

//       canvas1 -> Clear();

//       // Deleting the TMultiGraph object will automatically delete the graphs.
//       // You should not delete the graphs when the TMultigraph is still active.
//       // (from http://root.cern.ch/root/html/TMultiGraph.html)

// //             if(dataR0) delete graph0;
// //             if(dataR1) delete graph1;
// //             if(dataR3) delete graph3;
// //             if(dataR4) delete graph4;
// //             if(dataR5) delete graph5;
// //             if(dataR6) delete graph6;
// //             if(dataR8) delete graph8;
// //             if(dataR9) delete graph9;

//       delete leg;
//       delete mg;
//       delete rootFile;
//     }
  }

  delete canvas1;
}

/**
 * @brief Function to plot 2-dimensional graphics showing one parameter vs layer number.
 *
 * This function plots the 2-dimensional plots for all the sites for a specific parameter versus
 * the layer number (each plot corresponds to a specific chamber). It stores the plots both
 * in ROOT format, in order to load them from the ROOT CINT C++ interpreter, and in EPS format.
 * 
 * The function creates the following hierarchy of directories:
 * <tt>
 * layer-root-files/&lt;site&gt;/&lt;serial-number&gt;/&lt;graphName&gt;.root
 * layer-graphics/&lt;site&gt;/&lt;serial-number&gt;/&lt;graphName&gt;.eps
 * </tt>
 *
 * @param tree ROOT @c TTree from where the plotted data are extracted.
 * @param siteNames Array with the names of the different sites. The names must have less than
 * four letters (this is the current format: {ALU, BMC, etc.}). This array can be filled with
 * the function @c fillSites.
 * @param numSites The number of sites in @c tree.
 * @param parameter Parameter that will be plotted.
 * @param graphName Name the plotted graph will have.
 * @param title Title the plotted graph will have.
 * @param unit Units that were used to measure the plotted parameter.
 */
void twoDimPlotLayer(TTree* tree, Char_t siteNames[][4], UInt_t numSites, Char_t parameter[],
		     Char_t graphName[], Char_t title[], Char_t unit[]) {
  Char_t serialNumberChar[5];
  Char_t drawCommand[256];
  Char_t dirCommand[256];
  Char_t graphNameComplete[256];
  Char_t graphTitle[256];
  Char_t axisTitle[256];
  Char_t printName[256];
  Char_t cutCommand[256];
  UInt_t numChambers, indexSite, indexChamber, numEntries;
  UInt_t serialNumber, serialNumbers[50];
  TEventList* myList;
  TFile* rootFile;
  Char_t rootFileName[256];
  TMultiGraph* mg;
  TLegend* leg;
  Double_t nomValue;
  Double_t upperLimit = 0, lowerLimit = 0;
  TGraph *graph0 = 0, *graph0b = 0, *graph0c = 0, *graph1 = 0, *graph1b = 0, *graph1c = 0,
    *graph3 = 0, *graph3b = 0, *graph3c = 0, *graph4 = 0, *graph4b = 0, *graph4c = 0;
  UInt_t highestMeasurId;
  Char_t highestMeasurIdChar[5];

  // This canvas will contain the histograms of all chambers for a particular site)
  TCanvas* canvas1 = new TCanvas("canvas1");

  canvas1 -> SetLeftMargin(0.18);

  tree -> SetBranchAddress("SerialNumber", &serialNumber);

  // Loop around all sites
  for(indexSite = 0; indexSite < numSites; indexSite++) {

    // First we calculate how many chambers the tree has for the current site, and store the
    // serial numbers of them
    strcpy(cutCommand, "site == \"");
    strcat(cutCommand, siteNames[indexSite]);
    strcat(cutCommand, "\"");

    tree -> Draw(">>myList", cutCommand);

    myList = (TEventList*) gDirectory -> Get("myList");

    tree -> SetEventList(myList);
    
    // We fill the array serialNumbers

    numEntries = myList -> GetN();

    tree -> GetEntry(myList -> GetEntry(0));

    serialNumbers[0] = serialNumber;

    numChambers = 1;

    for(UInt_t i = 1; i < numEntries; i++) {
      tree -> GetEntry(myList -> GetEntry(i));
 
      if(serialNumber != serialNumbers[numChambers - 1]) {
	serialNumbers[numChambers] = serialNumber;
 	numChambers++;
      }
    }

    tree -> SetEventList(0);

    // Once we know the serial numbers of the chambers for the current site, we can begin
    // plotting one graph per chamber
    for(indexChamber = 0; indexChamber < numChambers; indexChamber++) {

      // We create a hierarchy of directories to hold the .eps and .root files
      strcpy(dirCommand, "mkdir -p layer-graphics/");
      strcat(dirCommand, siteNames[indexSite]);
      strcat(dirCommand, "/");
      sprintf(serialNumberChar, "%d", serialNumbers[indexChamber]);
      strcat(dirCommand, serialNumberChar);
      system(dirCommand);

      strcpy(dirCommand, "mkdir -p layer-root-files/");
      strcat(dirCommand, siteNames[indexSite]);
      strcat(dirCommand, "/");
      strcat(dirCommand, serialNumberChar);
      system(dirCommand);

      // We set the plot scale (this might be changed in case the tree has more records)
      if(strcmp("LayerPitch", parameter) == 0) {
	nomValue = 30.035;
	upperLimit = nomValue + 0.005;
	lowerLimit = nomValue - 0.001;
      }

      else { // Sag
	upperLimit = 0.15;
	lowerLimit = -0.15;
      }

      strcpy(graphNameComplete, graphName);
      strcat(graphNameComplete, "-");
      strcat(graphNameComplete, siteNames[indexSite]);
      strcat(graphNameComplete, "-");
      sprintf(serialNumberChar, "%d", serialNumbers[indexChamber]);
      strcat(graphNameComplete, serialNumberChar);

      // We set the first argument for the TTree::Draw member function (the parameter we want
      // to plot)
      strcpy(drawCommand, parameter);
      strcat(drawCommand, ":layer");

      strcpy(graphTitle, title);
      strcat(graphTitle, " (");
      strcat(graphTitle, siteNames[indexSite]);
      strcat(graphTitle, "-");
      strcat(graphTitle, serialNumberChar);
      strcat(graphTitle, ")");

      strcpy(axisTitle, title);
      strcat(axisTitle, " (");
      strcat(axisTitle, unit);
      strcat(axisTitle, ")");

      mg = new TMultiGraph(graphNameComplete, graphTitle);
      leg = new TLegend(0.79, 0.11, 0.89, 0.11);
      leg -> SetFillColor(10);

      // We set the second argument for the TTree:Draw member function (the selection criteria)
      strcpy(cutCommand, "site == \"");
      strcat(cutCommand, siteNames[indexSite]);
      strcat(cutCommand, "\" && serialNumber == ");
      sprintf(serialNumberChar, "%d", serialNumbers[indexChamber]);
      strcat(cutCommand, serialNumberChar);
      strcat(cutCommand, " && orientation == \"r0\" && HV == 1");

      // We must select the highest measurement id in order to plot only these values
      tree -> Draw("MeasurementId", cutCommand, "goff");
      highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
      sprintf(highestMeasurIdChar, "%d", highestMeasurId);
      strcat(cutCommand, " && measurementId == ");
      strcat(cutCommand, highestMeasurIdChar);
      strcat(cutCommand, " && badScan == 0");

      // We create the graph
      tree -> Draw(drawCommand, cutCommand, "goff");

      if(tree -> GetSelectedRows() > 0) {
	graph0 = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
	graph0 -> SetMarkerStyle(20);
	graph0 -> SetMarkerSize(1);
	graph0 -> SetMarkerColor(kRed);

	mg -> Add(graph0);
	leg -> SetY2(leg -> GetY2() + 0.025);
	leg -> AddEntry(graph0, "r0 HV", "p");
      }

      // We set the second argument for the TTree:Draw member function (the selection criteria)
      strcpy(cutCommand, "site == \"");
      strcat(cutCommand, siteNames[indexSite]);
      strcat(cutCommand, "\" && serialNumber == ");
      sprintf(serialNumberChar, "%d", serialNumbers[indexChamber]);
      strcat(cutCommand, serialNumberChar);
      strcat(cutCommand, " && orientation == \"r0\" && RO == 1");

      // We must select the highest measurement id in order to plot only these values
      tree -> Draw("MeasurementId", cutCommand, "goff");
      highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
      sprintf(highestMeasurIdChar, "%d", highestMeasurId);
      strcat(cutCommand, " && measurementId == ");
      strcat(cutCommand, highestMeasurIdChar);
      strcat(cutCommand, " && badScan == 0");

      // We create the graph
      tree -> Draw(drawCommand, cutCommand, "goff");

      if(tree -> GetSelectedRows() > 0) {
	graph0b = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
	graph0b -> SetMarkerStyle(24);
	graph0b -> SetMarkerSize(1);
	graph0b -> SetMarkerColor(kBlue);

	mg -> Add(graph0b);
	leg -> SetY2(leg -> GetY2() + 0.025);

	leg -> AddEntry(graph0b, "r0 RO", "p");
      }

      // We set the second argument for the TTree:Draw member function (the selection criteria)
      strcpy(cutCommand, "site == \"");
      strcat(cutCommand, siteNames[indexSite]);
      strcat(cutCommand, "\" && serialNumber == ");
      sprintf(serialNumberChar, "%d", serialNumbers[indexChamber]);
      strcat(cutCommand, serialNumberChar);

      if(strcmp(siteNames[indexSite], "WAS") == 0)
	strcat(cutCommand, " && orientation == \"r0\" && section == \"p9\"");

      else
	strcat(cutCommand, " && orientation == \"r0\" && section == \"p5\"");

      // We must select the highest measurement id in order to plot only these values
      tree -> Draw("MeasurementId", cutCommand, "goff");
      highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
      sprintf(highestMeasurIdChar, "%d", highestMeasurId);
      strcat(cutCommand, " && measurementId == ");
      strcat(cutCommand, highestMeasurIdChar);

      // We create the graph
      tree -> Draw(drawCommand, cutCommand, "goff");

      if(tree -> GetSelectedRows() > 0) {
	graph0c = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
	graph0c -> SetMarkerStyle(28);
	graph0c -> SetMarkerSize(1);
	graph0c -> SetMarkerColor(kGreen);

	mg -> Add(graph0c);
	leg -> SetY2(leg -> GetY2() + 0.025);

	if(strcmp(siteNames[indexSite], "WAS") == 0)
	  leg -> AddEntry(graph0c, "r0 p9", "p");

	else
	  leg -> AddEntry(graph0c, "r0 p5", "p");
      }

      // We set the second argument for the TTree:Draw member function (the selection criteria)
      strcpy(cutCommand, "site == \"");
      strcat(cutCommand, siteNames[indexSite]);
      strcat(cutCommand, "\" && serialNumber == ");
      strcat(cutCommand, serialNumberChar);
      strcat(cutCommand, " && orientation == \"r1\" && HV == 1");

      // We must select the highest measurement id in order to plot only these values
      tree -> Draw("MeasurementId", cutCommand, "goff");
      highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
      sprintf(highestMeasurIdChar, "%d", highestMeasurId);
      strcat(cutCommand, " && measurementId == ");
      strcat(cutCommand, highestMeasurIdChar);

      // We create the graph
      tree -> Draw(drawCommand, cutCommand, "goff");

      if(tree -> GetSelectedRows() > 0) {
	graph1 = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
	graph1 -> SetMarkerStyle(21);
	graph1 -> SetMarkerSize(1);
	graph1 -> SetMarkerColor(kYellow);

	mg -> Add(graph1);
	leg -> SetY2(leg -> GetY2() + 0.025);
	leg -> AddEntry(graph1, "r1 HV", "p");
      }

      // We set the second argument for the TTree:Draw member function (the selection criteria)
      strcpy(cutCommand, "site == \"");
      strcat(cutCommand, siteNames[indexSite]);
      strcat(cutCommand, "\" && serialNumber == ");
      strcat(cutCommand, serialNumberChar);
      strcat(cutCommand, " && orientation == \"r1\" && RO == 1");

      // We must select the highest measurement id in order to plot only these values
      tree -> Draw("MeasurementId", cutCommand, "goff");
      highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
      sprintf(highestMeasurIdChar, "%d", highestMeasurId);
      strcat(cutCommand, " && measurementId == ");
      strcat(cutCommand, highestMeasurIdChar);

      // We create the graph
      tree -> Draw(drawCommand, cutCommand, "goff");

      if(tree -> GetSelectedRows() > 0) {
	graph1b = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
	graph1b -> SetMarkerStyle(25);
	graph1b -> SetMarkerSize(1);
	graph1b -> SetMarkerColor(kMagenta);

	mg -> Add(graph1b);
	leg -> SetY2(leg -> GetY2() + 0.025);
	leg -> AddEntry(graph1b, "r1 RO", "p");
      }

      // We set the second argument for the TTree:Draw member function (the selection criteria)
      strcpy(cutCommand, "site == \"");
      strcat(cutCommand, siteNames[indexSite]);
      strcat(cutCommand, "\" && serialNumber == ");
      sprintf(serialNumberChar, "%d", serialNumbers[indexChamber]);
      strcat(cutCommand, serialNumberChar);

      if(strcmp(siteNames[indexSite], "WAS") == 0)
	strcat(cutCommand, " && orientation == \"r1\" && section == \"p9\"");

      else
	strcat(cutCommand, " && orientation == \"r1\" && section == \"p5\"");

      // We must select the highest measurement id in order to plot only these values
      tree -> Draw("MeasurementId", cutCommand, "goff");
      highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
      sprintf(highestMeasurIdChar, "%d", highestMeasurId);
      strcat(cutCommand, " && measurementId == ");
      strcat(cutCommand, highestMeasurIdChar);
      strcat(cutCommand, " && badScan == 0");

      // We create the graph
      tree -> Draw(drawCommand, cutCommand, "goff");

      if(tree -> GetSelectedRows() > 0) {
	graph1c = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
	graph1c -> SetMarkerStyle(29);
	graph1c -> SetMarkerSize(1);
	graph1c -> SetMarkerColor(28);

	mg -> Add(graph1c);
	leg -> SetY2(leg -> GetY2() + 0.025);

	if(strcmp(siteNames[indexSite], "WAS") == 0)
	  leg -> AddEntry(graph1c, "r1 p9", "p");

	else
	  leg -> AddEntry(graph1c, "r1 p5", "p");
      }

      // We set the second argument for the TTree:Draw member function (the selection criteria)
      strcpy(cutCommand, "site == \"");
      strcat(cutCommand, siteNames[indexSite]);
      strcat(cutCommand, "\" && serialNumber == ");
      strcat(cutCommand, serialNumberChar);
      strcat(cutCommand, " && orientation == \"r3\" && HV == 1");

      // We must select the highest measurement id in order to plot only these values
      tree -> Draw("MeasurementId", cutCommand, "goff");
      highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
      sprintf(highestMeasurIdChar, "%d", highestMeasurId);
      strcat(cutCommand, " && measurementId == ");
      strcat(cutCommand, highestMeasurIdChar);

      // We create the graph
      tree -> Draw(drawCommand, cutCommand, "goff");

      if(tree -> GetSelectedRows() > 0) {
	graph3 = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
	graph3 -> SetMarkerStyle(22);
	graph3 -> SetMarkerSize(1);
	graph3 -> SetMarkerColor(kGreen);

	mg -> Add(graph3);
	leg -> SetY2(leg -> GetY2() + 0.025);
	leg -> AddEntry(graph3, "r3 HV", "p");
      }

      // We set the second argument for the TTree:Draw member function (the selection criteria)
      strcpy(cutCommand, "site == \"");
      strcat(cutCommand, siteNames[indexSite]);
      strcat(cutCommand, "\" && serialNumber == ");
      strcat(cutCommand, serialNumberChar);
      strcat(cutCommand, " && orientation == \"r3\" && RO == 1");

      // We must select the highest measurement id in order to plot only these values
      tree -> Draw("MeasurementId", cutCommand, "goff");
      highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
      sprintf(highestMeasurIdChar, "%d", highestMeasurId);
      strcat(cutCommand, " && measurementId == ");
      strcat(cutCommand, highestMeasurIdChar);
      strcat(cutCommand, " && badScan == 0");

      // We create the graph
      tree -> Draw(drawCommand, cutCommand, "goff");

      if(tree -> GetSelectedRows() > 0) {
	graph3b = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
	graph3b -> SetMarkerStyle(26);
	graph3b -> SetMarkerSize(1);
	graph3b -> SetMarkerColor(kGreen);

	mg -> Add(graph3b);
	leg -> SetY2(leg -> GetY2() + 0.025);
	leg -> AddEntry(graph3b, "r3 RO", "p");
      }

      // We set the second argument for the TTree:Draw member function (the selection criteria)
      strcpy(cutCommand, "site == \"");
      strcat(cutCommand, siteNames[indexSite]);
      strcat(cutCommand, "\" && serialNumber == ");
      sprintf(serialNumberChar, "%d", serialNumbers[indexChamber]);
      strcat(cutCommand, serialNumberChar);

      if(strcmp(siteNames[indexSite], "WAS") == 0)
	strcat(cutCommand, " && orientation == \"r3\" && section == \"p9\"");

      else
	strcat(cutCommand, " && orientation == \"r3\" && section == \"p5\"");

      // We must select the highest measurement id in order to plot only these values
      tree -> Draw("MeasurementId", cutCommand, "goff");
      highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
      sprintf(highestMeasurIdChar, "%d", highestMeasurId);
      strcat(cutCommand, " && measurementId == ");
      strcat(cutCommand, highestMeasurIdChar);
      strcat(cutCommand, " && badScan == 0");

      // We create the graph
      tree -> Draw(drawCommand, cutCommand, "goff");

      if(tree -> GetSelectedRows() > 0) {
	graph3c = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
	graph3c -> SetMarkerStyle(30);
	graph3c -> SetMarkerSize(1);
	graph3c -> SetMarkerColor(kGreen);

	mg -> Add(graph3c);
	leg -> SetY2(leg -> GetY2() + 0.025);
	leg -> AddEntry(graph3c, "r3 p5", "p");

	if(strcmp(siteNames[indexSite], "WAS") == 0)
	  leg -> AddEntry(graph3c, "r3 p9", "p");

	else
	  leg -> AddEntry(graph3c, "r3 p5", "p");
      }

      // We set the second argument for the TTree:Draw member function (the selection criteria)
      strcpy(cutCommand, "site == \"");
      strcat(cutCommand, siteNames[indexSite]);
      strcat(cutCommand, "\" && serialNumber == ");
      strcat(cutCommand, serialNumberChar);
      strcat(cutCommand, " && orientation == \"r4\" && HV == 1");

      // We must select the highest measurement id in order to plot only these values
      tree -> Draw("MeasurementId", cutCommand, "goff");
      highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
      sprintf(highestMeasurIdChar, "%d", highestMeasurId);
      strcat(cutCommand, " && measurementId == ");
      strcat(cutCommand, highestMeasurIdChar);
      strcat(cutCommand, " && badScan == 0");

      // We create the graph
      tree -> Draw(drawCommand, cutCommand, "goff");

      if(tree -> GetSelectedRows() > 0) {
	graph4 = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
	graph4 -> SetMarkerStyle(23);
	graph4 -> SetMarkerSize(1);
	graph4 -> SetMarkerColor(kYellow);

 	mg -> Add(graph4);
	leg -> SetY2(leg -> GetY2() + 0.025);
	leg -> AddEntry(graph4, "r4 HV", "p");
      }

      // We set the second argument for the TTree:Draw member function (the selection criteria)
      strcpy(cutCommand, "site == \"");
      strcat(cutCommand, siteNames[indexSite]);
      strcat(cutCommand, "\" && serialNumber == ");
      strcat(cutCommand, serialNumberChar);
      strcat(cutCommand, " && orientation == \"r4\" && RO == 1");

      // We must select the highest measurement id in order to plot only these values
      tree -> Draw("MeasurementId", cutCommand, "goff");
      highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
      sprintf(highestMeasurIdChar, "%d", highestMeasurId);
      strcat(cutCommand, " && measurementId == ");
      strcat(cutCommand, highestMeasurIdChar);
      strcat(cutCommand, " && badScan == 0");

      // We create the graph
      tree -> Draw(drawCommand, cutCommand, "goff");

      if(tree -> GetSelectedRows() > 0) {
	graph4b = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
	graph4b -> SetMarkerStyle(27);
	graph4b -> SetMarkerSize(1);
	graph4b -> SetMarkerColor(kYellow);

	mg -> Add(graph4b);
	leg -> SetY2(leg -> GetY2() + 0.025);
	leg -> AddEntry(graph4b, "r4 RO", "p");
      }

      // We set the second argument for the TTree:Draw member function (the selection criteria)
      strcpy(cutCommand, "site == \"");
      strcat(cutCommand, siteNames[indexSite]);
      strcat(cutCommand, "\" && serialNumber == ");
      sprintf(serialNumberChar, "%d", serialNumbers[indexChamber]);
      strcat(cutCommand, serialNumberChar);

      if(strcmp(siteNames[indexSite], "WAS") == 0)
	strcat(cutCommand, " && orientation == \"r4\" && section == \"p9\"");

      else
	strcat(cutCommand, " && orientation == \"r4\" && section == \"p5\"");

      // We must select the highest measurement id in order to plot only these values
      tree -> Draw("MeasurementId", cutCommand, "goff");
      highestMeasurId = (UInt_t) (tree -> GetV1()[tree -> GetSelectedRows() - 1]);
      sprintf(highestMeasurIdChar, "%d", highestMeasurId);
      strcat(cutCommand, " && measurementId == ");
      strcat(cutCommand, highestMeasurIdChar);
      strcat(cutCommand, " && badScan == 0");

      // We create the graph
      tree -> Draw(drawCommand, cutCommand, "goff");

      if(tree -> GetSelectedRows() > 0) {
	graph4c = new TGraph(tree -> GetSelectedRows(), tree -> GetV2(), tree -> GetV1());
	graph4c -> SetMarkerStyle(3);
	graph4c -> SetMarkerSize(1);
	graph4c -> SetMarkerColor(kYellow);

	mg -> Add(graph4c);
	leg -> SetY2(leg -> GetY2() + 0.025);

	if(strcmp(siteNames[indexSite], "WAS") == 0)
	  leg -> AddEntry(graph4c, "r4 p9", "p");

	else
	  leg -> AddEntry(graph4c, "r4 p5", "p");
      }

      // We set the name for the .root file and open it
      strcpy(rootFileName, "./layer-root-files/");
      strcat(rootFileName, siteNames[indexSite]);
      strcat(rootFileName, "/");
      strcat(rootFileName, serialNumberChar);
      strcat(rootFileName, "/");
      strcat(rootFileName, graphNameComplete);
      strcat(rootFileName, ".root");

      rootFile = new TFile(rootFileName, "RECREATE");

      mg -> SetMaximum(upperLimit);
      mg -> SetMinimum(lowerLimit);

      mg -> Draw("ap");

      mg -> GetXaxis() -> SetTitle("Layer");
      mg -> GetYaxis() -> SetTitle(axisTitle);
      mg -> GetYaxis() -> SetTitleOffset(2.0);

      leg -> Draw();

      canvas1 -> Update();

      mg -> Write();
      leg -> Write();

      // We set the name for the EPS file
      strcpy(printName, "./layer-graphics/");
      strcat(printName, siteNames[indexSite]);
      strcat(printName, "/");
      strcat(printName, serialNumberChar);
      strcat(printName, "/");
      strcat(printName, graphNameComplete);
      strcat(printName, ".eps");

      // We print the graph to an EPS file
      canvas1 -> Print(printName);

      // We close the .root file
      rootFile -> Close();

      canvas1 -> Clear();

      // Deleting the TMultiGraph object will automatically delete the graphs.
      // You should not delete the graphs when the TMultigraph is still active.
      // (from http://root.cern.ch/root/html/TMultiGraph.html)

      //       if(dataR0) delete graph0;
      //       if(dataR1) delete graph1;
      //       if(dataR3) delete graph3;
      //       if(dataR4) delete graph4;
      //       if(dataR5) delete graph5;
      //       if(dataR6) delete graph6;
      //       if(dataR8) delete graph8;
      //       if(dataR9) delete graph9;

      delete rootFile;
      delete leg;
      delete mg;
    }
  }

  delete canvas1;
}

/**
 * @brief Function to plot the layer sag vs poutre position.
 *
 * This function plots the 2-dimensional plots for all the sites for the layer sag versus
 * the layer number (each plot corresponds to a specific chamber). It stores the plots both
 * in ROOT format, in order to load them from the ROOT CINT C++ interpreter, and in EPS format.
 * 
 * The function creates the following hierarchy of directories:
 * <tt>
 * layer-root-files/&lt;site&gt;/&lt;serial-number&gt;/&lt;graphName&gt;.root
 * layer-graphics/&lt;site&gt;/&lt;serial-number&gt;/&lt;graphName&gt;.eps
 * </tt>
 * 
 * It also makes several 3D plots showing the chamber sag (one plot per chamber).
 *
 * @param tree ROOT @c TTree from where the plotted data are extracted.
 * @param siteNames Array with the names of the different sites. The names must have less than
 * four letters (this is the current format: {ALU, BMC, etc.}). This array can be filled with
 * the function @c fillSites.
 * @param numSites The number of sites in @c tree.
 * @param parameter Parameter that will be plotted.
 * @param graphName Name the plotted graph will have.
 * @param title Title the plotted graph will have.
 * @param unit Units that were used to measure the plotted parameter.
 * @param average If this parameter is @c false, all the points for the different layers are
 * plotted, if it is @c true, only the average of the values for the different layers is shown
 * (one line per orientation, showing the average value).
 */
void twoDimPlotSagsPoutre(TTree* tree, Char_t siteNames[][4], UInt_t numSites,
			  Char_t parameter[], Char_t graphName[], Char_t title[],
			  Char_t unit[], Bool_t average = false) {
  Char_t dirCommand[256], dir[256];
  Char_t graphNameComplete[256];
  Char_t graphTitle[256], title3D[256], titleNeutralLine[256], titleNeut2D[256];
  Char_t axisTitle[256];
  Char_t printName[256], printName3D[256], printNameNeut1D[256], printNameNeut2D[256];
  Char_t cutCommand[256];
  UInt_t numChambers, indexSite, indexChamber, numEntries, numScans, indexScan;
  UInt_t serialNumbers[50];
  TEventList* myList;
  TFile* rootFile;
  Char_t rootFileName[256];
  TLegend *leg, *legNeutral;
  TGraph *graphs[4][8], *graphsAvg[4], **graphAvgR0R1;
  TF1* fitting[4];
  TMultiGraph *mg, *mgNeutral;
  TH1F* neutralLine1D = 0;
  Char_t serialNumberChar[8];

  Double_t graphsArraysSags[4][8][9]; // 4 orientations, 8 layers, 9 sections
  Double_t graphsArraysPoutres[4][8][9]; // 4 orientations, 8 layers, 9 sections
  Double_t graphsArraysSagsAvg[4][9]; // 4 orientations, 9 sections (we are averaging the
                                      // different layers)
  Double_t graphArraySagsAvgR0R1[9];  // 9 sections
  UInt_t numElementsArrays[4][8];
  UInt_t numberGraphs = 0, numberMgNeutral = 0;

  Char_t site[4];
  Double_t sag[8];
  Char_t orientation[3];
  UInt_t measurementId;
  Char_t section[3];
  Double_t poutre;
  UInt_t serialNumber;
  UInt_t badScan;
  UInt_t nLayers;
  Int_t nTubes;
  Int_t ml[2];
  UInt_t HV, RO;

  Char_t legendText[256];

  // Initialization of the array to control the number of elements afterwards
  memset(numElementsArrays, 0, 4 * 8 * sizeof(UInt_t));

  // Initialization of the array to average the values of the layers
  memset(graphsArraysSagsAvg, 0, 4 * 9 * sizeof(Double_t));

  // This canvas will contain the histograms of all chambers for a particular site)
  TCanvas* canvas1 = new TCanvas("canvas1");

  canvas1 -> SetLeftMargin(0.15);

  tree -> SetBranchAddress("Site", site);
  tree -> SetBranchAddress("SerialNumber", &serialNumber);
  tree -> SetBranchAddress(parameter, sag);
  tree -> SetBranchAddress("Orientation", orientation);
  tree -> SetBranchAddress("MeasurementId", &measurementId);
  tree -> SetBranchAddress("Section", section);
  tree -> SetBranchAddress("Poutre", &poutre);
  tree -> SetBranchAddress("BadScan", &badScan);
  tree -> SetBranchAddress("NLayers", &nLayers);
  tree -> SetBranchAddress("ML", ml);
  tree -> SetBranchAddress("HV", &HV);
  tree -> SetBranchAddress("RO", &RO);
  tree -> SetBranchAddress("NTubes", &nTubes);

  // Loop around all sites
  for(indexSite = 0; indexSite < numSites; indexSite++) {
    numberMgNeutral = 0;

    // First we calculate how many chambers the tree has for the current site, and store the
    // serial numbers of them
    strcpy(cutCommand, "site == \"");
    strcat(cutCommand, siteNames[indexSite]);
    strcat(cutCommand, "\"");

    tree -> Draw(">>myList", cutCommand);

    myList = (TEventList*) gDirectory -> Get("myList");

    tree -> SetEventList(myList);

    // We fill the array serialNumbers
    numEntries = tree -> GetSelectedRows();

    tree -> GetEntry(myList -> GetEntry(0));

    serialNumbers[0] = serialNumber;

    numChambers = 1;

    for(UInt_t i = 1; i < numEntries; i++) {
      tree -> GetEntry(myList -> GetEntry(i));
 
      if(serialNumber != serialNumbers[numChambers - 1]) {
	serialNumbers[numChambers] = serialNumber;
 	numChambers++;
      }
    }

    tree -> SetEventList(0);

    sprintf(titleNeutralLine, "Average sag (%s) (neutral line)", siteNames[indexSite]);
    neutralLine1D = new TH1F("neutLine1D", titleNeutralLine, 40, -100e-3, 100e-3);
    graphAvgR0R1 = new (TGraph*)[numChambers];
    mgNeutral = new TMultiGraph();
    legNeutral = new TLegend(0.79, 0.11, 0.89, 0.11);

    legNeutral -> SetBorderSize(0);
    legNeutral -> SetFillColor(10);

    // Once we know the serial numbers of the chambers for the current site, we can begin
    // plotting one graph per chamber
    for(indexChamber = 0; indexChamber < numChambers; indexChamber++) {

      // We create a hierarchy of directories to hold the .eps and .root files
      sprintf(graphNameComplete, "%s-%s-%d", graphName, siteNames[indexSite],
	      serialNumbers[indexChamber]);
      sprintf(graphTitle, "%s (%s-%d)", title, siteNames[indexSite],
	      serialNumbers[indexChamber]);
      sprintf(axisTitle, "%s (%s)", title, unit);
      sprintf(dir, "poutre-root-files/%s/%d/", siteNames[indexSite],
	      serialNumbers[indexChamber]);

      sprintf(rootFileName, "%s%s.root", dir, graphNameComplete);
      sprintf(dirCommand, "mkdir -p %s", dir);
      system(dirCommand);

      sprintf(dir, "poutre-graphics/%s/%d/", siteNames[indexSite], serialNumbers[indexChamber]);
      sprintf(dirCommand, "mkdir -p %s", dir);
      system(dirCommand);

      sprintf(printName, "%s%s.eps", dir, graphNameComplete);

      // Now we have to select only the events in the tree for the current site and the current
      // serial number
      sprintf(cutCommand, "site == \"%s\" && serialNumber == %d", siteNames[indexSite],
	      serialNumbers[indexChamber]);
      tree -> Draw(">>myList", cutCommand);

      myList = (TEventList*) gDirectory -> Get("myList");
      tree -> SetEventList(myList);
  
      numScans = tree -> GetSelectedRows();

      // Loop over all scans, filling the corresponding values for the sags
      for(indexScan = 0; indexScan < numScans; indexScan++) {
	tree -> GetEntry(myList -> GetEntry(indexScan));

	if(badScan == 0)
	  for(UInt_t i = 0; i < 8; i++)
	    if(sag[i] > -500 && (strcmp(section, "p9") == 0 || HV == 1 || RO == 1)) {
	      graphsArraysPoutres[numOrientation(orientation)][i][numElementsArrays[numOrientation(orientation)][i]] = poutre;

	      graphsArraysSags[numOrientation(orientation)][i][numElementsArrays[numOrientation(orientation)][i]] = sag[i];

	      if(average) 
		graphsArraysSagsAvg[numOrientation(orientation)][numElementsArrays[numOrientation(orientation)][i]] += sag[i];

	      numElementsArrays[numOrientation(orientation)][i]++;
	    }
      }

      // We create a TMultiGraph to hold all the graphs and a TLegend
      mg = new TMultiGraph();
      leg = new TLegend(0.79, 0.11, 0.89, 0.11);
      leg -> SetFillColor(10);

      // We set the colors and the style for markers and lines and add the graphs to the
      // multigraph
      for(UInt_t i = 0; i < 4; i++)
	if(average) {
	  if(numElementsArrays[i][0] > 0) {
	    numberGraphs++;

	    for(UInt_t j = 0; j < 9; j++)
	      graphsArraysSagsAvg[i][j] /= (Double_t) nLayers * (ml[1] == 2 ? 2 : 1);

	    graphsAvg[i] = new TGraph(numElementsArrays[i][0], graphsArraysPoutres[i][0],
				      graphsArraysSagsAvg[i]);
	    graphsAvg[i] -> SetMarkerStyle(20 + i);
	    graphsAvg[i] -> SetMarkerSize(0.7);
	    graphsAvg[i] -> SetMarkerColor(i + 1);
	    graphsAvg[i] -> SetLineColor(i + 1);
	    graphsAvg[i] -> Fit("pol1", "Q");
	    fitting[i] = (TF1*) (graphsAvg[i] -> GetFunction("pol1"));
	    fitting[i] -> SetLineColor(i + 1);

	    mg -> Add(graphsAvg[i]);

	    if(i < 2)
	      sprintf(legendText, "r%d", i);

	    else 
	      sprintf(legendText, "r%d", i + 1);

	    leg -> SetY2(leg -> GetY2() + 0.025);
	    leg -> AddEntry(graphsAvg[i], legendText, "l");

	    // We draw a 3D reconstruction of the sag chamber
	    if(nTubes != -9999) {
	      TCanvas* canvas2 = new TCanvas("canvas2");
	      canvas2 -> SetLeftMargin(0.13);
	      canvas2 -> cd();

	      Double_t a = graphsAvg[i] -> GetFunction("pol1") -> GetParameter(1);
	      Double_t b = graphsAvg[i] -> GetFunction("pol1") -> GetParameter(0);

	      TF2* f2 = new TF2("f2", "y*y*[0]*([1]*x+[2])", 200, 1600, -(30.0 * nTubes / 2.0),
				(30.0 * nTubes / 2.0));
	      f2 -> SetParameter(0, 1.0 / ((30.0 * nTubes / 2.0) * (30.0 * nTubes / 2.0)));
	      f2 -> SetParameter(1, a);
	      f2 -> SetParameter(2, b);

	      sprintf(title3D, "Layer Sag (%s-%d) (r%d)", siteNames[indexSite],
		      serialNumbers[indexChamber], (i < 2) ? i : i + 1);

	      f2 -> SetTitle(title3D);
	      f2 -> GetXaxis() -> SetTitleOffset(1.5);
	      f2 -> GetXaxis() -> SetTitle("Poutre (mm)");

	      f2 -> GetYaxis() -> SetTitleOffset(1.8);
	      f2 -> GetYaxis() -> SetTitle("Chamber width (mm)");

	      f2 -> GetZaxis() -> SetTitleOffset(1.5);
	      f2 -> GetZaxis() -> SetTitle("Average sag (mm)");

	      f2 -> Draw("surf");

	      sprintf(printName3D, "poutre-graphics/%s/%d/Sag3D-%s-%d-r%d.eps",
		      siteNames[indexSite], serialNumbers[indexChamber],
		      siteNames[indexSite], serialNumbers[indexChamber],
		      (i < 2) ? i : i + 1);

	      canvas2 -> Print(printName3D);
	      canvas1 -> cd();
	      delete f2;
	      delete canvas2;
	    }
	  }
	}

	else {
	  for(UInt_t j = 0; j < 8; j++) {
	    if(numElementsArrays[i][j] > 0) {
	      numberGraphs++;
	      graphs[i][j] = new TGraph(numElementsArrays[i][j], graphsArraysPoutres[i][j],
					graphsArraysSags[i][j]);
	      graphs[i][j] -> SetMarkerStyle(20);
	      graphs[i][j] -> SetMarkerSize(0.7);
	      graphs[i][j] -> SetMarkerColor(j + 1);
	      graphs[i][j] -> SetLineColor(j + 1);
	      graphs[i][j] -> SetLineStyle(i + 1);
	      mg -> Add(graphs[i][j]);

	      if(i < 2)
		sprintf(legendText, "r%d layer %d", i, j + 1);

	      else
		sprintf(legendText, "r%d layer %d", i + 1, j + 1);

	      leg -> SetY2(leg -> GetY2() + 0.025);
	      leg -> AddEntry(graphs[i][j], legendText, "l");
	    }
	  }
	}

      if(numElementsArrays[0][0] > 0 && numElementsArrays[1][0] > 0 && nTubes != -9999) {
	for(UInt_t j = 0; j < numElementsArrays[0][0]; j++)
	  graphArraySagsAvgR0R1[j] = (graphsArraysSagsAvg[0][j] +
				      graphsArraysSagsAvg[1][j]) / 2.0;

	TCanvas* canvas2 = new TCanvas("canvas2");
	canvas2 -> SetLeftMargin(0.13);
	canvas2 -> cd();

	graphAvgR0R1[indexChamber] = new TGraph(numElementsArrays[0][0],
						graphsArraysPoutres[0][0],
						graphArraySagsAvgR0R1);

	graphAvgR0R1[indexChamber] -> SetMarkerColor(indexChamber + 1);
	graphAvgR0R1[indexChamber] -> SetMarkerStyle(indexChamber + 20);
	graphAvgR0R1[indexChamber] -> SetMarkerSize(0.7);
	graphAvgR0R1[indexChamber] -> SetLineColor(indexChamber + 1);

	mgNeutral -> Add(graphAvgR0R1[indexChamber]);
	sprintf(serialNumberChar, "%s-%d", siteNames[indexSite], serialNumbers[indexChamber]);
	legNeutral -> AddEntry(graphAvgR0R1[indexChamber], "Hey", "lp");
	legNeutral -> SetY1(legNeutral -> GetY1() + 0.025);

	graphAvgR0R1[indexChamber] -> Fit("pol1", "Q0");
	numberMgNeutral++;

	Double_t a = graphAvgR0R1[indexChamber] -> GetFunction("pol1") -> GetParameter(1);
	Double_t b = graphAvgR0R1[indexChamber] -> GetFunction("pol1") -> GetParameter(0);

	TF2* f2 = new TF2("f2", "y*y*[0]*([1]*x+[2])", 200, 1600, -(30.0 * nTubes / 2.0),
			  (30.0 * nTubes / 2.0));
	f2 -> SetParameter(0, 1.0 / ((30.0 * nTubes / 2.0) * (30.0 * nTubes / 2.0)));
	f2 -> SetParameter(1, a);
	f2 -> SetParameter(2, b);

	sprintf(title3D, "Layer Sag (%s-%d) (neutral line)", siteNames[indexSite],
		serialNumbers[indexChamber]);

	f2 -> SetTitle(title3D);
	f2 -> GetXaxis() -> SetTitleOffset(1.5);
	f2 -> GetXaxis() -> SetTitle("Poutre (mm)");

	f2 -> GetYaxis() -> SetTitleOffset(1.8);
	f2 -> GetYaxis() -> SetTitle("Chamber width (mm)");

	f2 -> GetZaxis() -> SetTitleOffset(1.5);
	f2 -> GetZaxis() -> SetTitle("Average sag (mm)");

	f2 -> Draw("surf");

	sprintf(printName3D, "poutre-graphics/%s/%d/Sag3D-%s-%d-neutline.eps",
		siteNames[indexSite], serialNumbers[indexChamber],
		siteNames[indexSite], serialNumbers[indexChamber]);

	canvas2 -> Print(printName3D);
	canvas1 -> cd();
	delete canvas2;

	neutralLine1D -> Fill(a * 1000);
      }

      tree -> SetEventList(0);

      // We draw the multigraph and save the result both in .root and .eps format
      if(numberGraphs > 0) {
	mg -> Draw("alp");
	leg -> Draw();
	mg -> SetTitle(graphTitle);
	mg -> GetYaxis() -> SetTitle(axisTitle);
	mg -> GetYaxis() -> SetTitleOffset(1.5);
	mg -> GetXaxis() -> SetTitle("Poutre (mm)");

	if(average) {
	  if(numElementsArrays[0][0] > 0) {
	    TPaveStats* stats0 = (TPaveStats*) graphsAvg[0] -> GetListOfFunctions()
	      -> FindObject("stats");
	    stats0 -> SetTextColor(1);
	    stats0 -> SetX1NDC(0.18);
	    stats0 -> SetX2NDC(0.38);
	    stats0 -> SetY1NDC(0.8);
	    stats0 -> SetY2NDC(0.9);
	    canvas1 -> Modified();
	  }

	  if(numElementsArrays[1][0] > 0) {
	    TPaveStats* stats1 = (TPaveStats*) graphsAvg[1] -> GetListOfFunctions()
	      -> FindObject("stats");
	    stats1 -> SetTextColor(2); 
	    stats1 -> SetX1NDC(0.70);
	    stats1 -> SetX2NDC(0.90);
	    stats1 -> SetY1NDC(0.8);
	    stats1 -> SetY2NDC(0.9);
	    canvas1 -> Modified();
	  }

	  if(numElementsArrays[2][0] > 0) {
	    TPaveStats* stats3 = (TPaveStats*) graphsAvg[2] -> GetListOfFunctions()
	      -> FindObject("stats");
	    stats3 -> SetTextColor(3);
	    stats3 -> SetX1NDC(0.18);
	    stats3 -> SetX2NDC(0.38);
	    stats3 -> SetY1NDC(0.1);
	    stats3 -> SetY2NDC(0.2);
	    canvas1 -> Modified();
	  }

	  if(numElementsArrays[3][0] > 0) {
	    TPaveStats* stats4 = (TPaveStats*) graphsAvg[3] -> GetListOfFunctions()
	      -> FindObject("stats");
	    stats4 -> SetTextColor(4);
	    stats4 -> SetX1NDC(0.59);
	    stats4 -> SetX2NDC(0.79);
	    stats4 -> SetY1NDC(0.1);
	    stats4 -> SetY2NDC(0.2);
	    canvas1 -> Modified();
	  }
	}

	canvas1 -> Update();

	rootFile = new TFile(rootFileName, "RECREATE");

	mg -> Write();
	leg -> Write();

	canvas1 -> Print(printName);
	canvas1 -> Clear();
	rootFile -> Close();
	delete rootFile;
      }

      numberGraphs = 0;
      memset(numElementsArrays, 0, 4 * 8 * sizeof(UInt_t));
      memset(graphsArraysSagsAvg, 0, 4 * 9 * sizeof(Double_t));
      delete leg;
      delete mg;
    }

    TCanvas* canvas3 = new TCanvas("canvas3");
    canvas3 -> SetLeftMargin(0.15);
    canvas3 -> cd();

    neutralLine1D -> GetXaxis() -> SetTitle("Average variation of sag (mrad)");
    neutralLine1D -> Draw();
    neutralLine1D -> Fit("gaus", "QL");

    sprintf(printNameNeut1D, "chamber-graphics/%s/1D/Sag-neutline-%s-1D.eps",
	    siteNames[indexSite], siteNames[indexSite]);

    canvas3 -> Print(printNameNeut1D);
    canvas1 -> cd();
    delete canvas3;

    if(numberMgNeutral > 0) {
      TCanvas* canvas4 = new TCanvas("canvas4");
      gStyle -> SetOptFit(0);

      canvas4 -> SetLeftMargin(0.15);
      canvas4 -> cd();

      mgNeutral -> Draw("alp");
      legNeutral -> Draw();

      sprintf(titleNeut2D, "Average Layer Sag (neut. line) (%s)", siteNames[indexSite]);
      mgNeutral -> SetTitle(titleNeut2D);
      mgNeutral -> GetXaxis() -> SetTitle("Poutre (mm)");
      mgNeutral -> GetYaxis() -> SetTitle("Average layer sag (mm)");
      mgNeutral -> GetYaxis() -> SetTitleOffset(1.3);

      canvas4 -> Update();

      sprintf(printNameNeut2D, "chamber-graphics/%s/2D/Sag-neutline-%s-2D.eps",
	      siteNames[indexSite], siteNames[indexSite]);

      canvas4 -> Print(printNameNeut2D);
      canvas1 -> cd();
      delete canvas4;
      gStyle -> SetOptFit();
    }

    delete neutralLine1D;
    delete legNeutral;
    delete mgNeutral;
    delete graphAvgR0R1;
  }

  delete canvas1;
}

/**
 * The @c main function just sets several output options (about the style of plots), fills the
 * array with the names of the sites and calls the plotting functions with several different
 * parameters to make some graphics.
 */
int main() {
  UInt_t numSites; // Number of sites
  Char_t siteNames[50][4]; // Array to store the name of the different sites

  // Global settings for style and statistics
  gROOT -> SetStyle("Plain");
  gStyle -> SetOptStat("emruo");
  gStyle -> SetOptFit();
  gStyle -> SetStatFormat("8.6g");

  // We open the file and load the tree
  TFile* f = new TFile("scan_summary.root");
  TTree* tree = (TTree*) f -> Get("T");

  numSites = fillSites(tree, siteNames);

//   // Plots for the average temperature
//   twoDimPlotPoutre(tree, siteNames, numSites, "TempAvg", "TempAvg", "Average temperature",
// 		   "#circC");
  
//   // Plots for the variance of temperature
//   twoDimPlotPoutre(tree, siteNames, numSites, "TempVar", "TempVar", "Variance of temperature",
// 		   "#circC^{2}");

//   // Plots for the std deviation of temperature
//   twoDimPlotPoutre(tree, siteNames, numSites, "TempStdDev", "TempStdDev",
// 		   "Std deviation of temperature", "#circC");

//   // Plots for multilayer distance
//   twoDimPlotPoutre(tree, siteNames, numSites, "DeltaY", "MLDist", "Multilayer distance", "mm");
  
//   // Plots for multilayer Z shift
//   twoDimPlotPoutre(tree, siteNames, numSites, "DeltaZ", "MLZShift", "Multilayer Z shift", "mm");

//   // Plots for multilayer angle
//   twoDimPlotPoutre(tree, siteNames, numSites, "DeltaAlpha * 1e6", "MLAngle", "Multilayer angle",
// 		   "#murad");

  // Plots for Z Pitch
  twoDimPlotPoutre(tree, siteNames, numSites, "ZPitch", "MLZPitch", "Multilayer Z pitch", "mm"); 
   
//   // Plots for Y Pitch
//   twoDimPlotPoutre(tree, siteNames, numSites, "YPitch", "MLYPitch", "Multilayer Y Pitch", "mm");

//   // Plots for Tomograph precision
//   twoDimPlotPoutre(tree, siteNames, numSites, "TomoPrecision", "TomoPrecision",
// 		   "Tomograph precision", "mm");
 
//   // Plots for Layer Pitch
//   twoDimPlotLayer(tree, siteNames, numSites, "LayerPitch", "LayerPitch",
// 		  "Layer pitch", "mm");

//   // Plots for Layer Sag
//   twoDimPlotLayer(tree, siteNames, numSites, "Sag", "LayerSag",
// 		  "Layer sag", "mm");

//  twoDimPlotSagsPoutre(tree, siteNames, numSites, "Sag", "LayerSagPoutre", "Layer Sag", "mm");
//  twoDimPlotSagsPoutre(tree, siteNames, numSites, "Sag", "AvgLayerSagPoutre",
// 		      "Average Layer Sag", "mm", true);

  // We close the file
  f -> Close();
  delete f;

  return 0;
}
